@@ -19,6 +19,8 @@ from yellowdog_client.model import (
     ComputeRequirementTemplateSummary,
     ComputeSourceTemplate,
     ComputeSourceTemplateSummary,
+    Instance,
+    InstanceSearch,
     Keyring,
     KeyringSummary,
     MachineImageFamilySearch,
@@ -36,85 +38,74 @@ from yellowdog_client.model import (
     WorkRequirementStatus,
     WorkRequirementSummary,
 )
-from yellowdog_client.model.exceptions import ObjectNotFoundException
 
-from yd_commands.args import ARGS_PARSER
-from yd_commands.config import unpack_namespace_in_prefix
 from yd_commands.interactive import select
 from yd_commands.object_utilities import (
     get_filtered_work_requirements,
     get_task_groups_from_wr_summary,
 )
 from yd_commands.printing import (
-    CONSOLE,
-    TABLE_OUTPUT_STYLE,
+    CONSOLE_TABLE,
     indent,
     print_error,
     print_log,
     print_numbered_object_list,
-    print_object_detail,
     print_yd_object,
     sorted_objects,
 )
-from yd_commands.wrapper import CLIENT, CONFIG_COMMON, main_wrapper
+from yd_commands.utils import unpack_namespace_in_prefix
+from yd_commands.wrapper import ARGS_PARSER, CLIENT, CONFIG_COMMON, main_wrapper
 
 
 @main_wrapper
 def main():
-    """
-    If multiple object types are selected, they're all listed.
-    """
     if not check_for_valid_option():
-        raise Exception("No object type options chosen")
+        raise Exception("Please choose a single, valid listing type")
 
     # Always use interactive mode for selections
     ARGS_PARSER.interactive = True
 
     if ARGS_PARSER.object_paths:
         list_object_paths()
-
-    if ARGS_PARSER.work_requirements or ARGS_PARSER.task_groups or ARGS_PARSER.tasks:
+    elif ARGS_PARSER.work_requirements or ARGS_PARSER.task_groups or ARGS_PARSER.tasks:
         list_work_requirements()
-
-    if ARGS_PARSER.worker_pools:
+    elif ARGS_PARSER.worker_pools:
         list_worker_pools()
-
-    if ARGS_PARSER.compute_requirements:
+    elif ARGS_PARSER.compute_requirements or ARGS_PARSER.instances:
         list_compute_requirements()
-
-    if ARGS_PARSER.compute_templates:
+    elif ARGS_PARSER.compute_templates:
         list_compute_templates()
-
-    if ARGS_PARSER.source_templates:
+    elif ARGS_PARSER.source_templates:
         list_source_templates()
-
-    if ARGS_PARSER.keyrings:
+    elif ARGS_PARSER.keyrings:
         list_keyrings()
-
-    if ARGS_PARSER.image_families:
+    elif ARGS_PARSER.image_families:
         list_image_families()
-
-    if ARGS_PARSER.namespaces:
+    elif ARGS_PARSER.namespaces:
         list_namespaces()
 
 
 def check_for_valid_option() -> bool:
     """
-    At least one of the listing options must be selected.
+    Only one of the listing options must be selected.
     """
-    return (
-        ARGS_PARSER.object_paths
-        or ARGS_PARSER.work_requirements
-        or ARGS_PARSER.task_groups
-        or ARGS_PARSER.tasks
-        or ARGS_PARSER.worker_pools
-        or ARGS_PARSER.compute_requirements
-        or ARGS_PARSER.compute_templates
-        or ARGS_PARSER.source_templates
-        or ARGS_PARSER.keyrings
-        or ARGS_PARSER.image_families
-        or ARGS_PARSER.namespaces
-    )
+    if [
+        ARGS_PARSER.object_paths,
+        ARGS_PARSER.work_requirements,
+        ARGS_PARSER.task_groups,
+        ARGS_PARSER.tasks,
+        ARGS_PARSER.worker_pools,
+        ARGS_PARSER.compute_requirements,
+        ARGS_PARSER.compute_templates,
+        ARGS_PARSER.source_templates,
+        ARGS_PARSER.keyrings,
+        ARGS_PARSER.image_families,
+        ARGS_PARSER.namespaces,
+        ARGS_PARSER.instances,
+    ].count(True) == 1:
+        return True
+    else:
+        return False
 
 
 def list_work_requirements():
@@ -126,16 +117,19 @@ def list_work_requirements():
     options chosen.
     """
     print_log(
-        f"Listing Work Requirements with  '{CONFIG_COMMON.namespace}'in namespace "
+        f"Listing Work Requirements with  '{CONFIG_COMMON.namespace}' in namespace "
         f"and '{CONFIG_COMMON.name_tag}' in tag",
     )
+    if ARGS_PARSER.active_only:
+        print_log("Listing active Work Requirements only")
+
     exclude_filter = (
         [
             WorkRequirementStatus.COMPLETED,
             WorkRequirementStatus.CANCELLED,
             WorkRequirementStatus.FAILED,
         ]
-        if ARGS_PARSER.live_only
+        if ARGS_PARSER.active_only
         else []
     )
     work_requirement_summaries: List[WorkRequirementSummary] = (
@@ -146,9 +140,17 @@ def list_work_requirements():
             exclude_filter=exclude_filter,
         )
     )
+    if len(work_requirement_summaries) == 0:
+        print_log("No matching Work Requirements")
+        return
+
     work_requirement_summaries = sorted_objects(work_requirement_summaries)
     if not (ARGS_PARSER.task_groups or ARGS_PARSER.tasks):
-        print_numbered_object_list(CLIENT, work_requirement_summaries)
+        if ARGS_PARSER.details:
+            for wr_summary in select(CLIENT, work_requirement_summaries):
+                print_yd_object(wr_summary)
+        else:
+            print_numbered_object_list(CLIENT, work_requirement_summaries)
     else:
         selected_work_summaries = select(
             CLIENT, work_requirement_summaries, single_result=True
@@ -164,7 +166,11 @@ def list_task_groups(work_summary: WorkRequirementSummary):
     )
     task_groups = sorted_objects(task_groups)
     if not ARGS_PARSER.tasks:
-        print_numbered_object_list(CLIENT, task_groups)
+        if ARGS_PARSER.details:
+            for task_group in select(CLIENT, task_groups):
+                print_yd_object(task_group)
+        else:
+            print_numbered_object_list(CLIENT, task_groups)
     else:
         task_groups = select(CLIENT, task_groups, single_result=True)
         for task_group in task_groups:
@@ -178,7 +184,11 @@ def list_tasks(task_group: TaskGroup, work_summary: WorkRequirementSummary):
     )
     tasks: List[Task] = CLIENT.work_client.find_tasks(task_search)
     tasks = sorted_objects(tasks)
-    print_numbered_object_list(CLIENT, tasks, parent=work_summary)
+    if ARGS_PARSER.details:
+        for task in select(CLIENT, tasks):
+            print_yd_object(task)
+    else:
+        print_numbered_object_list(CLIENT, tasks)
 
 
 def list_object_paths():
@@ -212,22 +222,18 @@ def list_object_paths():
         return
 
     # Print object details for selected objects
-    # Skip interactive selection if there are five or fewer results
-    if len(object_paths) > 5:
-        object_paths = select(CLIENT, object_paths, override_quiet=True)
+    object_paths = select(CLIENT, object_paths, override_quiet=True)
     if len(object_paths) != 0:
         print_log(f"Showing Object details for {len(object_paths)} Object(s)")
-    for index, object_path in enumerate(object_paths):
-        try:
-            if index == 0:
-                print()
-            object_detail: ObjectDetail = CLIENT.object_store_client.get_object_detail(
-                namespace=namespace, name=object_path.name
-            )
-            print_object_detail(object_detail)
-        except ObjectNotFoundException:
-            print_log(f"Note: '{object_path.name}' is a prefix not an object")
-        print()
+    for object_path in object_paths:
+        if object_path.prefix:
+            print_log(f"Object Path '{object_path.name}' is a prefix not an object")
+            continue
+        object_detail: ObjectDetail = CLIENT.object_store_client.get_object_detail(
+            namespace=namespace, name=object_path.name
+        )
+        # print_object_detail(object_detail)  # Retired for now
+        print_yd_object(object_detail)
 
 
 def list_worker_pools():
@@ -236,11 +242,13 @@ def list_worker_pools():
     )
     excluded_states = (
         [WorkerPoolStatus.TERMINATED, WorkerPoolStatus.SHUTDOWN]
-        if ARGS_PARSER.live_only
+        if ARGS_PARSER.active_only
         else []
     )
-    if ARGS_PARSER.live_only:
+    showing_all = True
+    if ARGS_PARSER.active_only:
         print_log("Displaying active Worker Pools only")
+        showing_all = False
     worker_pool_summaries = [
         wp_summary
         for wp_summary in worker_pool_summaries
@@ -251,7 +259,7 @@ def list_worker_pools():
         return
     if ARGS_PARSER.details:
         for worker_pool_summary in select(
-            CLIENT, sorted_objects(worker_pool_summaries), showing_all=True
+            CLIENT, sorted_objects(worker_pool_summaries), showing_all=showing_all
         ):
             worker_pool: WorkerPool = CLIENT.worker_pool_client.get_worker_pool_by_id(
                 worker_pool_summary.id
@@ -259,7 +267,7 @@ def list_worker_pools():
             print_yd_object(worker_pool)
     else:
         print_numbered_object_list(
-            CLIENT, sorted_objects(worker_pool_summaries), showing_all=True
+            CLIENT, sorted_objects(worker_pool_summaries), showing_all=showing_all
         )
 
 
@@ -270,6 +278,9 @@ def list_compute_requirements():
         f" names containing '{CONFIG_COMMON.name_tag}'"
     )
 
+    if ARGS_PARSER.active_only:
+        print_log("Listing active Compute Requirements only")
+
     cr_search = ComputeRequirementSearch(
         namespace=CONFIG_COMMON.namespace,
     )
@@ -281,7 +292,7 @@ def list_compute_requirements():
     filtered_compute_requirements: List[ComputeRequirement] = []
     excluded_states = (
         [ComputeRequirementStatus.TERMINATED, ComputeRequirementStatus.TERMINATING]
-        if ARGS_PARSER.live_only
+        if ARGS_PARSER.active_only
         else []
     )
     for compute_requirement in compute_requirements:
@@ -295,10 +306,45 @@ def list_compute_requirements():
         ):
             filtered_compute_requirements.append(compute_requirement)
 
-    print_numbered_object_list(
-        CLIENT,
-        sorted_objects(filtered_compute_requirements),
+    if len(filtered_compute_requirements) == 0:
+        print_log("No matching Compute Requirements")
+        return
+
+    filtered_compute_requirements = sorted_objects(filtered_compute_requirements)
+
+    if ARGS_PARSER.instances:
+        for compute_requirement in select(
+            CLIENT, filtered_compute_requirements, single_result=True
+        ):
+            list_instances(compute_requirement)
+        return
+
+    if ARGS_PARSER.details:
+        for compute_requirement in select(CLIENT, filtered_compute_requirements):
+            print_yd_object(compute_requirement)
+    else:
+        print_numbered_object_list(CLIENT, filtered_compute_requirements)
+
+
+def list_instances(compute_requirement: ComputeRequirement):
+    """
+    List the instances within a Compute Requirement.
+    """
+    instance_search: InstanceSearch = InstanceSearch(
+        computeRequirementId=compute_requirement.id
     )
+    search_client: SearchClient = CLIENT.compute_client.get_instances(
+        instance_search=instance_search
+    )
+    instances: List[Instance] = search_client.list_all()
+    if len(instances) == 0:
+        print_log("No instances to list")
+        return
+    if ARGS_PARSER.details:
+        for instance in select(CLIENT, search_client.list_all()):
+            print_yd_object(instance)
+    else:
+        print_numbered_object_list(CLIENT, search_client.list_all())
 
 
 def list_compute_templates():
@@ -327,9 +373,6 @@ def list_compute_templates():
         print_numbered_object_list(CLIENT, sorted_objects(cr_templates))
         return
 
-    print_log(
-        "Please select Compute Requirement Template(s) for which to obtain details"
-    )
     cr_templates = select(CLIENT, cr_templates)
     for cr_template in cr_templates:
         cr_template_detail: ComputeRequirementTemplate = (
@@ -364,7 +407,6 @@ def list_source_templates():
         print_numbered_object_list(CLIENT, sorted_objects(cs_templates))
         return
 
-    print_log("Please select Compute Source Template(s) for which to obtain details")
     cs_templates = select(CLIENT, sorted_objects(cs_templates))
     deserialisation_failures = []  # Temporary bug workaround
     for cs_template in cs_templates:
@@ -400,7 +442,6 @@ def list_keyrings():
         print_numbered_object_list(CLIENT, sorted_objects(keyrings))
         return
 
-    print_log("Please select Keyring(s) for which to obtain details")
     keyrings = select(CLIENT, keyrings)
     for keyring_summary in keyrings:
         print_yd_object(get_keyring(keyring_summary.name))
@@ -435,7 +476,6 @@ def list_image_families():
         print_numbered_object_list(CLIENT, sorted_objects(image_family_summaries))
         return
 
-    print_log("Please select Image Families for which to obtain details")
     for image_family_summary in select(CLIENT, sorted_objects(image_family_summaries)):
         image_family = CLIENT.images_client.get_image_family_by_id(
             image_family_summary.id
@@ -498,9 +538,8 @@ def list_namespaces():
         ]
     )
     print()
-    CONSOLE.print(
+    CONSOLE_TABLE.print(
         indent(tabulate(rows, headings, tablefmt="simple_outline")),
-        style=TABLE_OUTPUT_STYLE,
     )
     print()
 
